﻿using System;
using System.Windows.Forms;

namespace SleepTimer
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // 残り時間の現在値
        private TimeSpan Remaining;

        // タイマーの状態
        private bool IsTicking = false;

        private void ButtonStartStop_Click(object sender, EventArgs e)
        {
            // タイマーが動いていない時
            if(!IsTicking)
            {
                // まだ計測が開始されていない場合
                if (!buttonReset.Enabled)
                {
                    // 入力した時間を残り時間に設定
                    Remaining = new TimeSpan(int.Parse(textBoxHours.Text), int.Parse(textBoxMinutes.Text), int.Parse(textBoxSeconds.Text));

                    // 残り時間のテキストボックスを無効化
                    textBoxHours.Enabled = false;
                    textBoxMinutes.Enabled = false;
                    textBoxSeconds.Enabled = false;

                    // リセットボタンを有効化
                    buttonReset.Enabled = true;
                }

                // タイマー開始
                timerSleep.Start();

                // 開始ボタン → 一時停止ボタン
                buttonStartStop.Text = "一時停止";

                // タイマーの状態を動作中に変更
                IsTicking = true;
            }
            // 一時停止ボタン押下時
            else
            {
                // タイマーを一時停止
                timerSleep.Stop();

                // 一時停止ボタン → 再開ボタン
                buttonStartStop.Text = "再開";

                // タイマーの状態を停止中に変更
                IsTicking = false;
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // 状態の初期化
            ResetAll();
        }

        private void TimerSleep_Tick(object sender, EventArgs e)
        {
            // 残り時間を1秒減らす
            Remaining -= new TimeSpan(0, 0, 1);

            // テキストボックス更新
            textBoxHours.Text = Remaining.Hours.ToString();
            textBoxMinutes.Text = Remaining.Minutes.ToString();
            textBoxSeconds.Text = Remaining.Seconds.ToString();

            // 残り時間が0を下回ったら、スリープする
            if(Remaining.TotalSeconds < 0)
            {
                // 状態の初期化
                ResetAll();
                // スリープへ移行
                Application.SetSuspendState(PowerState.Suspend, false, false);
            }
        }

        private void ButtonReset_Click(object sender, EventArgs e)
        {
            // 状態の初期化
            ResetAll();
        }

        /// <summary>
        /// 状態の初期化
        /// </summary>
        private void ResetAll()
        {
            // タイマー停止
            timerSleep.Stop();

            // 「開始」ボタンに設定
            buttonStartStop.Text = "開始";

            // タイマーの状態を停止中に変更
            IsTicking = false;

            // 残り時間をデフォルトに戻す
            TimeSpan RemDefault = new TimeSpan(0, 30, 0);
            textBoxHours.Text = RemDefault.Hours.ToString();
            textBoxMinutes.Text = RemDefault.Minutes.ToString();
            textBoxSeconds.Text = RemDefault.Seconds.ToString();

            // 残り時間のテキストボックスを有効化
            textBoxHours.Enabled = true;
            textBoxMinutes.Enabled = true;
            textBoxSeconds.Enabled = true;

            // リセットボタンを無効化
            buttonReset.Enabled = false;
        }
    }
}
